#!/bin/bash
# Die Adresse der Fritzbox
# Beispiele:
# fbox="https://192.168.178.1:49140" # SSL-Verschlüsselt, Port 444
# fbox="http://192.168.178.1" # http normal, Port 80

#################
# Konfiguration #
fbox="http://192.168.178.1"

# Bei https, Ausnahme für selbst signiertes Zertifikat, sonst leer lassen
INSECURE= 
#INSECURE="--insecure" 

# das bei der Fritzbox konfigurierte Passwort
# unter System->FRITZ!Box-Benutzer->Anmeldung im Heimnetz
# Ist die Option "Anmeldung mit dem FRITZ!Box-Kennwort" aktiv
# genügt das Passwort für die Anmeldung
PASSWD="geheim" 

# Der Benutzernamen für die Fritzbox
# Dieser lässt sich unter "System -> FRITZ!Box-Benutzer" erstellen.
# Dann kann man die Option
# "Anmeldung mit FRITZ!Box-Benutzernamen und Kennwort" aktivieren

USER=""
# Vollständige Anzeige: full
# Leer lassen für schnellere Ergebnisse
# "min" gibt nur "OK" zurück.
#VIEW=
VIEW="full"
#VIEW="min"
#####################

if [ "$fbox" = "" ]; then
echo "Content-type: text/html"
echo ""
echo "Bitte Fritzbox Adresse im Script eintragen"
exit 1
fi

CURL=$(which curl) 
MD5SUM=$(which md5sum)
ICONV=$(which iconv)
AWK=$(which awk)

STYLE_RED="<font color=\"red\">"
STYLE_GREEN="<font color=\"green\">"
STYLE_YELOW="<font color=\"yellow\">"
STYLE_CYAN="<font color=\"cyan\">"
STYLE_BLUE="<font color=\"blue\">"
STYLE_END="</font>"

CURLCMD="$CURL $INSECURE -s $fbox/webservices/homeautoswitch.lua"

get_sid() {
SID=$($CURL $INSECURE -s $fbox/login_sid.lua | sed 's/.*<SID>\(.*\)<\/SID>.*/\1/')

if [ "$SID" = "0000000000000000" ]; then
challenge=$($CURL $INSECURE -s $fbox/login_sid.lua |  grep -o "<Challenge>[a-z0-9]\{8\}" | cut -d'>' -f 2)
if [ "$VIEW" != "min" ]; then
echo "Challenge: $challenge"
echo "<br />"
fi

CPSTR="$challenge-$PASSWD"
hash=`echo -n $CPSTR | $ICONV -f ISO8859-1 -t UTF-16LE | $MD5SUM -b | $AWK '{print substr($0,1,32)}'`
if [ "$VIEW" != "min" ]; then
echo "MD5: $hash"
echo "<br />"
fi

RESPONSE="$challenge-$hash"
POSTDATA="username=$USER&response=$RESPONSE"
SID=$($CURL $INSECURE --data "$POSTDATA" -s $fbox/login_sid.lua | sed 's/.*<SID>\(.*\)<\/SID>.*/\1/')
fi
if [ "$VIEW" != "min" ]; then
echo "SID: $SID"
echo "<br />"
fi
}

get_value() {
RESULT=""
if [ "$2" = "" ]; then
RESULT=$($CURLCMD"?sid=$SID&switchcmd=$1")
else
RESULT=$($CURLCMD"?sid=$SID&ain=$2&switchcmd=$1")
fi
}

echo "Content-type: text/html"
echo ""
echo "<html><head><title>Fritzbox Smarthome steuern</title></head>"
echo "<body>"
if [ "$VIEW" != "min" ]; then
echo 
echo "<input type=\"button\" name=\"reload\" onClick=\"location.assign(window.location.pathname)\" value=\"Seite neu laden, aktuelle Werte ermitteln\">"
echo "<br />"
fi

# Bei der Fritzbox anmelden
get_sid
if [ "$SID" = "0000000000000000" ]; then
echo "$STYLE_RED Anmeldung fehlgeschlagen $STYLE_END"
echo "<br />"
exit 1
fi

if [ "$SID" = "" ]; then
echo "$STYLE_RED Anmeldung fehlgeschlagen $STYLE_END"
echo "<br />"
exit 1
fi
if [ "$VIEW" != "min" ]; then
echo "$STYLE_GREEN Anmeldung erfolgreich $STYLE_END"
echo "<br />"
fi

# Liste der Schalter ermitteln
get_value getswitchlist
ainlist=$RESULT

# Formular auswerten / Schalter umchalten
if [ "$QUERY_STRING" != "" ]; then
#echo $QUERY_STRING
THEAIN=`echo "$QUERY_STRING" | sed -n 's/^.*ain=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
CMD=`echo "$QUERY_STRING" | sed -n 's/^.*subbtn=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`

if [ "$CMD" == "Einschalten" ]
then
  switch=$($CURLCMD"?ain=$THEAIN&switchcmd=setswitchon&sid=$SID&")
fi  

if [ "$CMD" == "Ausschalten" ]
then
    switch=$($CURLCMD"?ain=$THEAIN&switchcmd=setswitchoff&sid=$SID&")
fi    

if [ "$CMD" == "Umschalten" ]
then
  switch=$($CURLCMD"?ain=$THEAIN&switchcmd=setswitchtoggle&sid=$SID&")
fi   

fi

if [ "$VIEW" = "min" ]
then
echo "OK"
else

# Werte der Schalter holen
echo "AIN list: $ainlist"
echo "<br />"
echo "<br />"
COUNTER=0
IFS=', ' read -r -a array <<< "$ainlist"
for AIN in "${array[@]}"
do
let COUNTER=COUNTER+1 
echo "$STYLE_RED Actor #$COUNTER $STYLE_END"
echo "<br />"
switchpresent=0
get_value getswitchname $AIN
echo "$STYLE_BLUE Name: $RESULT $STYLE_END"
echo "<br />"
if [ "$VIEW" = "full" ]; then
  get_value getswitchpresent $AIN
  echo "Connected: $RESULT"
  echo "<br />"
  if [ "$RESULT" = "1" ]; then
   get_value getswitchstate $AIN
   echo "State: $RESULT"
   echo "<br />"
   get_value getswitchpower $AIN
   switchpower=`awk "BEGIN {printf \"%.2f\n\", $RESULT/1000}"`
   echo "Power: $switchpower W"
   echo "<br />"
   get_value getswitchenergy $AIN
   echo "Energy: $RESULT Wh"
  fi
fi
echo "<br />"
echo "<form action=\"smart_fritz_server.sh\" method=\"get\">"
echo "<input type=\"text\" name=\"ain\" value=\"$AIN\"></input>"
echo "<input type=\"submit\" name=\"subbtn\" value=\"Umschalten\">"
echo "<input type=\"submit\" name=\"subbtn\" value=\"Einschalten\">"
echo "<input type=\"submit\" name=\"subbtn\" value=\"Ausschalten\">"
echo "</form>"


#fi
echo "<br />"
done
fi
echo "</body></html>" 
